# python_project_sample
© 2022 だえうホームページ All rights reserved.

# コード解説
```
#coding:utf-8　
import tkinter
import tkinter.messagebox
```

## キャンバスのサイズ
```
CANVAS_SIZE = 400
```

## マスの数
```
NUM_SQUARE = 8
```

## 色
```python
BOARD_COLOR = 'lightblue' # 盤面の背景色
YOUR_COLOR = 'pink' # 自分の石の色
COM_COLOR = 'lightgreen' # 相手の石の色
PLACABLE_COLOR = 'red' # 次に石を置ける場所の色
```

## プレイヤーを示す値
```python
YOU = 1
COM = 2

class Othello():
    def __init__(self, master):

        self.master = master 
        self.player = YOU # 次に置く石の色
        self.board = None 
        self.color = { # 石の色を保持するやつ
            YOU : YOUR_COLOR,
            COM : COM_COLOR
        }
```

### ウィジェットの作成
        self.createWidgets()

### イベントの設定
        self.setEvents()

### オセロゲームの初期化
        self.initOthello()


    def createWidgets(self):

## キャンバスの作成
```python
        self.canvas = tkinter.Canvas(
            self.master,
            bg=BOARD_COLOR,
            width=CANVAS_SIZE+1,
            height=CANVAS_SIZE+1,
            highlightthickness=0
        )
        self.canvas.pack(padx=10, pady=10)

    def setEvents(self):

        self.canvas.bind('<ButtonPress>', self.click)

    def initOthello(self):

        self.board = [[None] * NUM_SQUARE for i in range(NUM_SQUARE)]
```

### １マスのサイズ
        self.square_size = CANVAS_SIZE // NUM_SQUARE

### マスを描画
        for y in range(NUM_SQUARE):
            for x in range(NUM_SQUARE):
#### 長方形の開始・終了座標を計算
                xs = x * self.square_size
                ys = y * self.square_size
                xe = (x + 1) * self.square_size
                ye = (y + 1) * self.square_size
                
#### 長方形を描画
                tag_name = 'square_' + str(x) + '_' + str(y)
                self.canvas.create_rectangle(
                    xs, ys,
                    xe, ye,
                    tag=tag_name
                )

### 石の描画位置を計算
```python
        your_init_pos_1_x = NUM_SQUARE // 2
        your_init_pos_1_y = NUM_SQUARE // 2
        your_init_pos_2_x = NUM_SQUARE // 2 - 1
        your_init_pos_2_y = NUM_SQUARE // 2 - 1

        your_init_pos = (
            (your_init_pos_1_x, your_init_pos_1_y),
            (your_init_pos_2_x, your_init_pos_2_y)
        )
```

### 計算した描画位置に石を描画
        for x, y in your_init_pos:
            self.drawDisk(x, y, self.color[YOU])

### 対戦相手の石の描画位置を計算
        com_init_pos_1_x = NUM_SQUARE // 2 - 1
        com_init_pos_1_y = NUM_SQUARE // 2
        com_init_pos_2_x = NUM_SQUARE // 2
        com_init_pos_2_y = NUM_SQUARE // 2 - 1

        com_init_pos = (
            (com_init_pos_1_x, com_init_pos_1_y),
            (com_init_pos_2_x, com_init_pos_2_y)
        )

### 計算した描画位置に石を描画
        for x, y in com_init_pos:
            self.drawDisk(x, y, self.color[COM])

### 最初に置くことができる石の位置
        placable = self.getPlacable()

        self.showPlacable(placable)

    def drawDisk(self, x, y, color):

### マスの中心座標
        center_x = (x + 0.5) * self.square_size
        center_y = (y + 0.5) * self.square_size

### 中心座標から石の開始座標と終了座標を計算
        xs = center_x - (self.square_size * 0.8) // 2
        ys = center_y - (self.square_size * 0.8) // 2
        xe = center_x + (self.square_size * 0.8) // 2
        ye = center_y + (self.square_size * 0.8) // 2
        
### 四角を描画する
```python
        tag_name = 'disk_' + str(x) + '_' + str(y)
        self.canvas.create_rectangle(
            xs, ys,
            xe, ye,
            fill=color,
            tag=tag_name
        )

        self.board[y][x] = color

    def getPlacable(self):

        placable = []

        for y in range(NUM_SQUARE):
            for x in range(NUM_SQUARE):
                if self.checkPlacable(x, y):
                    placable.append((x, y))

        return placable

    def checkPlacable(self, x, y):

        if self.board[y][x] != None:
            return False

        if self.player == YOU:
            other = COM
        else:
            other = YOU

        for j in range(-1, 2):
            for i in range(-1, 2):

                if i == 0 and j == 0:
                    continue

                if x + i < 0 or x + i >= NUM_SQUARE or y + j < 0 or y + j >= NUM_SQUARE:
                    continue

                if self.board[y + j][x + i] != self.color[other]:
                    continue

                for s in range(2, NUM_SQUARE):
                    if x + i * s >= 0 and x + i * s < NUM_SQUARE and y + j * s >= 0 and y + j * s < NUM_SQUARE:
                        
                        if self.board[y + j * s][x + i * s] == None:
                            
                            break

                        if self.board[y + j * s][x + i * s] == self.color[self.player]:
                            return True
        return False

    def showPlacable(self, placable):

        for y in range(NUM_SQUARE):
            for x in range(NUM_SQUARE):
```

#### 石が置けるマスの色を変更
```python
                tag_name = 'square_' + str(x) + '_' + str(y)
                if (x, y) in placable:
                    self.canvas.itemconfig(
                        tag_name,
                        fill=PLACABLE_COLOR
                    )

                else:
                    self.canvas.itemconfig(
                        tag_name,
                        fill=BOARD_COLOR
                    )

    def click(self, event):

        if self.player != YOU:
            return

        x = event.x // self.square_size
        y = event.y // self.square_size

        if self.checkPlacable(x, y):
```

#### 次に石を置けるところ
            self.place(x, y, self.color[self.player])

    def place(self, x, y, color):

### 石が置かれた時に裏返す
        self.reverse(x, y)

### 石を置く
        self.drawDisk(x, y, color)

### 次に石を置くプレイヤー
```python
        before_player = self.player
        self.nextPlayer()
        
        if before_player == self.player:
            if self.player != YOU:
                tkinter.messagebox.showinfo('結果', 'スキップ')
            else:
                tkinter.messagebox.showinfo('結果', 'スキップ')

        elif not self.player:
```

#### 次に置けるプレイヤーがいない場合ゲーム終了
            self.showResult()
            return

### 石がおける位置
```python
        placable = self.getPlacable()
        self.showPlacable(placable)

        if self.player == COM:
            self.master.after(1000, self.com)

    def reverse(self, x, y):

        if self.board[y][x] != None:
```

#### (x,y)にすでに石が置かれている場合は何もしない
```python
            return

        if self.player == COM:
            other = YOU
        else:
            other = COM

        for j in range(-1, 2):
            for i in range(-1, 2):
                if i == 0 and j == 0:
                    continue

                if x + i < 0 or x + i >= NUM_SQUARE or y + j < 0 or y + j >= NUM_SQUARE:
                    continue

                if self.board[y + j][x + i] != self.color[other]:
                    continue

                for s in range(2, NUM_SQUARE):
                # 盤面外のマスはチェックしない
                    if x + i * s >= 0 and x + i * s < NUM_SQUARE and y + j * s >= 0 and y + j * s < NUM_SQUARE:
                        
                        if self.board[y + j * s][x + i * s] == None:
                            break

                        if self.board[y + j * s][x + i * s] == self.color[self.player]:
                            for n in range(1, s):
                                self.board[y + j * n][x + i * n] = self.color[self.player]

                                tag_name = 'disk_' + str(x + i * n) + '_' + str(y + j * n)
                                self.canvas.itemconfig(
                                    tag_name,
                                    fill=self.color[self.player]
                                )
                            
                            break
```

    def nextPlayer(self):

        before_player = self.player

### 石を置くプレイヤーを切り替える
```python
        if self.player == YOU:
            self.player = COM
        else:
            self.player = YOU
```

### 切り替え後のプレイヤーが石を置けるかどうかを確認
        placable = self.getPlacable()

        if len(placable) == 0:
#### 置けなければスキップ
            self.player = before_player

#### 置けるかどうか
            placable = self.getPlacable()

            if len(placable) == 0:
                self.player = None

    def showResult(self):

### それぞれの色の石の数を数える
```python
        num_your = 0
        num_com = 0

        for y in range(NUM_SQUARE):
            for x in range(NUM_SQUARE):
                if self.board[y][x] == YOUR_COLOR:
                    num_your += 1
                elif self.board[y][x] == COM_COLOR:
                    num_com += 1
```
        
### 結果
```python
        tkinter.messagebox.showinfo('結果', 'あなた' + str(num_your) + '：COM' + str(num_com))

    def com(self):
```

### 石が置けるマスを取得
        placable = self.getPlacable()

### 最初のマスを次に石を置くマスに
        x, y = placable[0]

### 石を置く
        self.place(x, y, COM_COLOR)

## スクリプト処理
```python
app = tkinter.Tk()
app.title('othello')
othello = Othello(app)
app.mainloop()
```

# 引用元
[このサイト](https://daeudaeu.com/tkinter-othello/)からダウンロード

## 変更点
```
・盤面の色を緑から水色にした
・石の色を黒と白からピンクと黄緑にした
・次におけるところを示す色を黄色から赤にした
・石の形を丸から四角にした
```

# Copyright and License Information

This project is licensed under the terms of the MIT license.

